#ifndef K3DSDK_SHADER_COLLECTION_H
#define K3DSDK_SHADER_COLLECTION_H

// K-3D
// Copyright (c) 1995-2005, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\author Tim Shead (tshead@k-3d.com)
*/

#include "signal_system.h"
#include "sl.h"

namespace k3d
{

/////////////////////////////////////////////////////////////////////////////
// shader_collection

/// Loads and caches metadata for a collection of shaders
/** \deprecated We only need this for backwards-compatibility with pre-0.5.0.39 documents */
class shader_collection
{
public:
	shader_collection();

	/// Defines a signal emitted to display human-readable progress messages while loading shaders
	typedef sigc::signal1<void, std::string> message_signal_t;
	message_signal_t& message_signal();

	void load_shaders(const std::string& SourcePaths);

	const sl::shaders_t& shaders();

private:
	message_signal_t m_message_signal;
	sl::shaders_t m_shaders;
};

} // namespace k3d

#endif // K3DSDK_SHADER_COLLECTION_H
